/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.dataIdentificationSettings.DataIdentification;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Aggregiert die LZD waehrend einem bestimmten Intervall, diese Klasse
 * ermoeglicht die Kommunikation mit dem DatenVerteiler und der Hauptklasse der
 * SWEinheit, die konkrete Methode der Rechnungen sollen in vererbten Klassen
 * definiert werden.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public abstract class AbstraktAggregator implements ClientSenderInterface, SBArchivAnfrageBearbeiter {

	/**
	 * Stringkonstanten.
	 */
	protected static final String TYP_LANGZEITZAEHLSTELLE = "typ.langZeitZählstelle";

	/** Attributnamen für Verkehrsdaten. */
	protected static final String[] FZ_ATTRIB = new String[] { "Kfz", "PkwÄ", "LkwÄ", "KfzNk", "PkwG", "Pkw", "Krad",
		"Lfw", "PkwA", "Lkw", "Bus", "LkwK", "LkwA", "SattelKfz" };

	/** Attributname für "Wert". */
	protected static final String STR_WERT = "Wert";
	/** Attributname für "Güte". */
	protected static final String STR_GUETE = "Güte";
	/** Attributname für "Index". */
	protected static final String STR_INDEX = "Index";
	/** Attributname für "Verfahren". */
	protected static final String STR_VERFAHREN = "Verfahren";

	/**
	 * Zeitpunkt, ermoeglicht den beginn des Tages auszurechnen.
	 */
	private static Calendar calendar = Calendar.getInstance();
	/**
	 * Fuer Fehlremeldungem.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Datenbeschreibung für Fahrzeuganzahl Agreggierte Daten.
	 */
	private DataDescription ddAggregierteLzd;
	/**
	 * Die LangZeitZaehstelle fuer die die Daten Ausgerechnet werden sollen.
	 */
	private Collection<SystemObject> objekteLzz;
	/**
	 * Verbindung zum DAV.
	 */
	private ClientDavInterface dav;
	/**
	 * Mapt System Objekt der LZZ zur zugehoerigen Struktur mit ergaenzenden
	 * Daten.
	 */
	private final HashMap<SystemObject, LZZDaten> mapSoLzzZumLZZDaten = new LinkedHashMap<>();

	/**
	 * Einfache Struktur um die ergaenzenden Daten zu eine LZZ speichern.
	 *
	 */
	protected class LZZDaten {
		/**
		 * Der aggregierte Datensatz, wird bei erzeugung geschickt.
		 */
		private Data ergebnissDatenSatz;
		/**
		 * ZeitStempel letztes DatenSatzes im Archiv.
		 */
		private long zeitStempelLetztesDS = AbfrPufferKonstanten.ZEIT_BEGINN;
		/**
		 * Fuer jedes Parameter merken wir, ob wir schon einen ermittelbaren
		 * Wert bekommen haben.
		 */
		private final boolean[] ermittelbarWertEingekommen = new boolean[AbstraktAggregator.FZ_ATTRIB.length];
		/**
		 * Ob die Aggregierte LZD zu Archivierung eingestellt sind.
		 */
		private boolean archiviert;
		/**
		 * Anzahl der Aggregierten DatenSaetze.
		 */
		private long anzahlDatenSaetze;
		/**
		 * Ob die LZZ angemeldet ist fuer das Absenden der Aggregierten DS.
		 *
		 */
		private boolean angemeldet;

		/**
		 * Erfragt ob der die LZZ angemeldet sit fuer absenden der AGG DS.
		 *
		 * @return true, wenn angemeldet
		 */
		public boolean isAngemeldet() {
			return angemeldet;
		}

		/**
		 * Initialisiert die Daten.
		 */
		public void resetDaten() {
			for (int i = 0; i < AbstraktAggregator.FZ_ATTRIB.length; i++) {
				ermittelbarWertEingekommen[i] = false;
			}
			anzahlDatenSaetze = 0;
		}

		/**
		 * Erfragt nach dem Array von Flags, fuer welche Paramter haben wir
		 * schon einen ermittelbaren Wert bekommen.
		 *
		 * @return Array Ermitellbar Flags
		 */
		public boolean[] getErmittelbarWert() {
			return ermittelbarWertEingekommen;
		}

		/**
		 * Legt fest, ob die LZZ angemeldet ist duer Absenden der aggregierten
		 * DS.
		 *
		 * @param angemeldet
		 *            der gewünschte Zustand
		 */
		public void setAngemeldet(final boolean angemeldet) {
			this.angemeldet = angemeldet;
		}

		/**
		 * Inkrementiert die Anzahl der DatenSaetze.
		 *
		 * @return Die neue Anzahl der DatenSaetze
		 */
		public long incrementAnzahlDatenSaetze() {
			return ++anzahlDatenSaetze;
		}

		/**
		 * Erfragt die Anzahl der DatenSaetze.
		 *
		 * @return die Anzahl
		 */
		public long getAnzahlDatenSaetze() {
			return anzahlDatenSaetze;
		}

		/**
		 * Setzt die Anzahl der Datensaetze.
		 *
		 * @param anzahlDatenSaetze
		 *            die Anzahl
		 */
		public void setAnzahlDatenSaetze(final long anzahlDatenSaetze) {
			this.anzahlDatenSaetze = anzahlDatenSaetze;
		}

		/**
		 * Erfragt den Ergebniss Datensatz.
		 *
		 * @return der Datensatz
		 */
		public Data getErgebnissDatenSatz() {
			return ergebnissDatenSatz;
		}

		/**
		 * Legt fest den Datensatz fuer Ergebnisse.
		 *
		 * @param datenSatz
		 *            der neue Datensatz
		 */
		public void setErgebnissDatenSatz(final Data datenSatz) {
			this.ergebnissDatenSatz = datenSatz;
		}

		/**
		 * Erfragt den Zeitstempel des Letzten DS.
		 *
		 * @return der ZeitStempel
		 */
		public long getZeitStempelLetztesDS() {
			return zeitStempelLetztesDS;
		}

		/**
		 * Legt fest den Zeitstempel des Letzten DS.
		 *
		 * @param zeitStempel
		 *            der Zeitstempel
		 */
		public void setZeitStempelLetztesDS(final long zeitStempel) {
			this.zeitStempelLetztesDS = zeitStempel;
		}

		/**
		 * Initialisiert dem ZeitStempel d.h. uberschreibnt nur wenn es noch
		 * nicht initialisiert wurde.
		 *
		 * @param zeitStempel
		 *            der Zeitstempel
		 */
		public void initZeitStempelLetztesDS(final long zeitStempel) {
			if (this.zeitStempelLetztesDS == AbfrPufferKonstanten.ZEIT_BEGINN) {
				this.zeitStempelLetztesDS = zeitStempel;
			}
		}

		/**
		 * Erfragt ob die Aggregierte Daten von der LZZ gespeichert werden.
		 *
		 * @return true, wenn die Daten archiviert werden
		 */
		public boolean isArchiviert() {
			return archiviert;
		}

		/**
		 * Legt fest ob, die Daten archviert werden.
		 *
		 * @param archiviert
		 *            true, wenn sie gespeichert werden sollen
		 */
		public void setArchiviert(final boolean archiviert) {
			this.archiviert = archiviert;
		}
	}

	/**
	 * Wird beim jeden einkommenden Datensatz gerufen.
	 *
	 * @param objekt
	 *            LZZ
	 * @param datensatz
	 *            Daten
	 * @param zeitStempel
	 *            zeitStempel
	 */
	public abstract void bearbeiteDatenSatz(SystemObject objekt, Data datensatz, long zeitStempel);

	/**
	 * In dieser Methode soll jeder konkreter Aggregator Beschreibung seiner
	 * Daten erzeugen und die Map von SystemObjekt LZZ zum eigenen
	 * DatenStrukturen.
	 *
	 * @param asp
	 *            Aspekt der erzeugenden Daten
	 * @param initialDav
	 *            DatenVerteilerverbindung
	 * @param objekteLZZ
	 *            Menge aller LZZ
	 * @return Die Datenbeschreibung der Daten, die erzeugt werden
	 */
	public abstract DataDescription init(Aspect asp, ClientDavInterface initialDav,
			Collection<SystemObject> objekteLZZ);

	/**
	 * Initialisation, unter anderen stellt den Erfassungsintervall ein, liest
	 * aus dem Archiv ab, zu welcher Zeitpunkt wurden die letzten Daten im
	 * Archiv eingeschrieben.
	 *
	 * @param asp
	 *            Aspekt der erzeugenden Daten
	 * @param initialDav
	 *            DatenVerteilerverbindung
	 * @param objekteLZZ
	 *            Menge aller LZZ
	 * @param zeitpunktJetzt
	 *            der initiale Startzeitpunkt
	 */
	public void initialisiere(final Aspect asp, final ClientDavInterface initialDav,
			final Collection<SystemObject> objekteLZZ, final long zeitpunktJetzt) {

		objekteLzz = objekteLZZ;
		this.dav = initialDav;
		ddAggregierteLzd = init(asp, initialDav, objekteLZZ);

		final StreamBasierteArchivAnfrage anfrage = new StreamBasierteArchivAnfrage(initialDav);

		anfrage.generiereAnfrage(objekteLzz, ddAggregierteLzd, true, 1, zeitpunktJetzt);
		if (anfrage.istErfolgreich()) {
			anfrage.bearbeiteAntwortStream(this);
		} else {
			for (final SystemObject obj : objekteLzz) {
				AbstraktAggregator.LOGGER
						.error("Fuer " + obj + " konnte der letzte aggregierte DS nicht bestimmt werden");
				final LZZDaten lzz = mapSoLzzZumLZZDaten.get(obj);
				if (lzz != null) {
					lzz.setZeitStempelLetztesDS(
							AbstraktAggregator.getTagesBeginnVonZeitStempel(System.currentTimeMillis()));
				}
			}
		}
	}

	/**
	 * Erzeugt einen Datensatz zum abschicken.
	 *
	 * @param objekt
	 *            LZZ fuer die der DatenSatz gehoert
	 * @param zeitStempelVon
	 *            Intervall Anfang
	 * @param zeitStempelBis
	 *            Intervall Ende
	 * @param ermittelbar
	 *            false, wenn alle Datensaetze nicht gekommen sind, sonst true
	 */
	public abstract void erzeugeErgebniss(SystemObject objekt, long zeitStempelVon, long zeitStempelBis,
			boolean ermittelbar);

	/**
	 * Sendet einen Datensatz, wenn noch nicht angemeldet ist fuer die Sendung,
	 * dann meldet sich erst an.
	 *
	 * @param objekt
	 *            Die LZZ
	 * @param resDaten
	 *            Datensatz zum senden
	 */
	public void sendeDatenSatz(final SystemObject objekt, final ResultData resDaten) {

		final LZZDaten lzz = mapSoLzzZumLZZDaten.get(objekt);

		if (!lzz.isAngemeldet()) {
			anmeldeUndSende(resDaten);
			lzz.setAngemeldet(true);
			return;
		}

		try {
			dav.sendData(resDaten);
		} catch (final DataNotSubscribedException e) {
			AbstraktAggregator.LOGGER.error("Fehler bei Sendung eines DatenSatzes " + e.getMessage());
			e.printStackTrace();
		} catch (final SendSubscriptionNotConfirmed e) {
			AbstraktAggregator.LOGGER.error("Fehler bei Sendung eines DatenSatzes " + e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Meldet sich als Sender fuer die Aggregierte Daten an und gleich schickt
	 * einen DatenSatz ab.
	 *
	 * @param resultData
	 *            der zu versendende Datensatz
	 *
	 */
	public void anmeldeUndSende(final ResultData resultData) {

		try {
			dav.subscribeSource(this, resultData);
		} catch (final OneSubscriptionPerSendData e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Meldet sich als Sender fuer die Aggregierte Daten ab.
	 *
	 */
	public void abmeldeAlsSender() {
		for (final SystemObject lzz : this.mapSoLzzZumLZZDaten.keySet()) {
			final LZZDaten lzzDt = mapSoLzzZumLZZDaten.get(lzz);
			if (lzzDt.isAngemeldet()) {
				dav.unsubscribeSender(this, lzz, ddAggregierteLzd);
			}
		}
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		// Standardimplementierung tut nichts
	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		return false;
	}

	/**
	 * Bearbeitet die Daten aus der StreamBasierten Archivanfrage, wird benutzt
	 * um den Letzten ZeitStempel der Aggregierten Daten aus dem Archiv
	 * auslesen.
	 */
	@Override
	public void bearbeiteArchivDaten(final SystemObject objekt, final DataDescription datenBeschreibung,
			final DataState status, final Data daten, final long zeitStempel) {

		if (status != DataState.END_OF_ARCHIVE) {
			final LZZDaten lzz = mapSoLzzZumLZZDaten.get(objekt);
			if (lzz.getZeitStempelLetztesDS() > zeitStempel) {
				return;
			}
			lzz.setZeitStempelLetztesDS(zeitStempel);
		}
	}

	/**
	 * Setzt den Zeitpunkt der Daten ein, die als Startpunkt benutzt werden,
	 * wenn es noch keine Aggregierte Daten im Archiv gibt.
	 *
	 * @param lzz
	 *            LZZ
	 * @param zeitPunkt
	 *            ZeitStempel
	 */
	public void initLetzteAggregierteDatenZS(final SystemObject lzz, final long zeitPunkt) {
		final LZZDaten lzzStruct = mapSoLzzZumLZZDaten.get(lzz);
		lzzStruct.initZeitStempelLetztesDS(zeitPunkt);
	}

	/**
	 * Setzt den Zeitpunkt der letzten Aggregierten Daten ein.
	 *
	 * @param lzz
	 *            LZZ
	 * @param zeitPunkt
	 *            ZeitStempel
	 */
	public void setLetzteAggregierteDatenZS(final SystemObject lzz, final long zeitPunkt) {
		final LZZDaten lzzObj = mapSoLzzZumLZZDaten.get(lzz);
		if (lzzObj != null) {
			lzzObj.setZeitStempelLetztesDS(zeitPunkt);
		}
	}

	/**
	 * Ergibt den Zeitpunkt der Letzten Aggregierten Daten.
	 *
	 * @param lzz
	 *            LZZ
	 * @return zeitStempel des Letzten Datensatzes mit agg. Daten
	 */
	public long getLetzteAggregierteDatenZS(final SystemObject lzz) {
		final LZZDaten lzzStruct = mapSoLzzZumLZZDaten.get(lzz);
		return lzzStruct.getZeitStempelLetztesDS();
	}

	/**
	 * Setzt dem Archiviertunsparameter der LZZ.
	 *
	 * @param lzz
	 *            System Objekt LZZ
	 * @param archiviert
	 *            true wenn daten archiviert werden
	 */
	public void setLZZArchiviert(final SystemObject lzz, final boolean archiviert) {
		final LZZDaten lzzObj = mapSoLzzZumLZZDaten.get(lzz);
		if (lzzObj != null) {
			lzzObj.setArchiviert(archiviert);
		}
	}

	/**
	 * Ergibt dem Archivierunsparameter der LZZ.
	 *
	 * @param lzz
	 *            System Objekt LZZ
	 * @return true, wenn daten archiviert werden
	 */
	public boolean isLZZArchiviert(final SystemObject lzz) {
		final LZZDaten lzzObj = mapSoLzzZumLZZDaten.get(lzz);
		if (lzzObj != null) {
			return lzzObj.isArchiviert();
		}
		return false;
	}

	/**
	 * Ein Datensatz mit geloeschten Einstellungen gekommen ist - Nicht mehr
	 * parametriert.
	 *
	 * @param dateiID
	 *            Datenbeschreibung der Einstellung
	 * @param alteDatei
	 *            Datensatz mit Werten der Einstellung
	 */
	public void bearbeiteGeloeschteEinstellung(final DataIdentification dateiID, final Data alteDatei) {

		if (!(dateiID.getObject().getType().getPid().equals(AbstraktAggregator.TYP_LANGZEITZAEHLSTELLE)
				&& dateiID.getDataDescription().getAttributeGroup().getPid()
						.equals(ddAggregierteLzd.getAttributeGroup().getPid())
				&& dateiID.getDataDescription().getAspect().getPid().equals(ddAggregierteLzd.getAspect().getPid()))) {
			return;
		}
		final SystemObject lzz = dateiID.getObject();
		setLZZArchiviert(lzz, false);
	}

	/**
	 * Ein Datensatz mit neuen Einstellungen gekommen ist - Neue Parametrierung.
	 *
	 * @param dateiID
	 *            Datenbeschreibung der Einstellung
	 * @param neueDatei
	 *            Datensatz mit Werten der Einstellung
	 */
	public void bearbeiteNeueEinstellung(final DataIdentification dateiID, final Data neueDatei) {

		if (!(dateiID.getObject().getType().getPid().equals(AbstraktAggregator.TYP_LANGZEITZAEHLSTELLE)
				&& dateiID.getDataDescription().getAttributeGroup().getPid()
						.equals(ddAggregierteLzd.getAttributeGroup().getPid())
				&& dateiID.getDataDescription().getAspect().getPid().equals(ddAggregierteLzd.getAspect().getPid()))) {
			return;
		}

		boolean archiviert = true;

		// Wenn es nich Archiviert werden soll
		if (neueDatei.getUnscaledValue(AbfrPufferKonstanten.ATR_ARCHIVIEREN).getState().getValue() != 1) {
			archiviert = false;
		}
		final SystemObject lzz = dateiID.getObject();
		setLZZArchiviert(lzz, archiviert);
	}

	/**
	 * Ein Datensatz mit veraenderten Einstellungen gekommen ist -
	 * Umparametrierung.
	 *
	 * @param dateiID
	 *            Datenbeschreibung der Einstellung
	 * @param neueDatei
	 *            Datensatz mit Werten der Einstellung
	 */
	public void bearbeiteVeraenderteEinstellung(final DataIdentification dateiID, final Data neueDatei) {

		if (!(dateiID.getObject().getType().getPid().equals(AbstraktAggregator.TYP_LANGZEITZAEHLSTELLE)
				&& dateiID.getDataDescription().getAttributeGroup().getPid()
						.equals(ddAggregierteLzd.getAttributeGroup().getPid())
				&& dateiID.getDataDescription().getAspect().getPid().equals(ddAggregierteLzd.getAspect().getPid()))) {
			return;
		}

		boolean archiviert = true;

		// Wenn es nich Archiviert werden soll
		if (neueDatei.getUnscaledValue(AbfrPufferKonstanten.ATR_ARCHIVIEREN).getState().getValue() != 1) {
			archiviert = false;
		}

		final SystemObject lzz = dateiID.getObject();
		setLZZArchiviert(lzz, archiviert);
	}

	/**
	 * Aufrundet den ZeitStempel so, dass man dem Beginn des Tages bekommt in
	 * dem der Zeitstepel liegt.
	 *
	 * @param zeitStempel
	 *            Ein Zeitstempel
	 * @return Beginn des Tages, in dem der ZeitStempel liegt
	 */
	public static long getTagesBeginnVonZeitStempel(final long zeitStempel) {
		AbstraktAggregator.calendar.setTimeInMillis(zeitStempel);
		AbstraktAggregator.calendar.set(Calendar.HOUR_OF_DAY, 0);
		AbstraktAggregator.calendar.set(Calendar.MINUTE, 0);
		AbstraktAggregator.calendar.set(Calendar.SECOND, 0);
		AbstraktAggregator.calendar.set(Calendar.MILLISECOND, 0);
		return AbstraktAggregator.calendar.getTimeInMillis();
	}

	/**
	 * Setzt nur einen Attribut (Wert + Guete) als nich ermittelbar.
	 *
	 * @param subData
	 *            Der Datensatz der nur ein Wert enthaelt
	 */
	public void setAttributNichtErmittelbar(final Data subData) {
		subData.getUnscaledValue(AbstraktAggregator.STR_WERT).set(-1);
		final Data gueteData = subData.getItem(AbstraktAggregator.STR_GUETE);
		gueteData.getUnscaledValue(AbstraktAggregator.STR_INDEX).set(-1);
		gueteData.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).set(0);
	}

	/**
	 * Initialsiert einen Datensatz.
	 *
	 * @param data
	 *            DatenSatz
	 */
	public void setLeer(final Data data) {
		for (final String element : AbstraktAggregator.FZ_ATTRIB) {
			final Data subData = data.getItem(element);
			subData.getUnscaledValue(AbstraktAggregator.STR_WERT).set(-1);
			final Data gueteData = subData.getItem(AbstraktAggregator.STR_GUETE);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_INDEX).set(-1);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).set(0);
		}
	}

	/**
	 * Setzt ein DatenSatz als nicht ermittelbar.
	 *
	 * @param data
	 *            DatenSatz
	 */
	public void setNichtErmittelbar(final Data data) {
		for (final String element : AbstraktAggregator.FZ_ATTRIB) {
			final Data subData = data.getItem(element);
			final Data gueteData = subData.getItem(AbstraktAggregator.STR_GUETE);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_INDEX).set(-1);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).set(0);
		}
	}

	/**
	 * Bsetimmt, ob fuer eine LZZ die aggregierte Daten archiviert werden.
	 *
	 * @param lzz
	 *            SystemObjekt LangZeitZaehlstelle
	 * @param aggMenge
	 *            Menge der Aggregatoren
	 * @return true, wenn keine von den Ausgaben der Aggregatoren archiviert
	 *         werden, sonst false
	 */
	public static boolean isKeinArchiviert(final SystemObject lzz, final Collection<AbstraktAggregator> aggMenge) {
		for (final AbstraktAggregator agg : aggMenge) {
			if (agg.isLZZArchiviert(lzz)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Copiert dem Wert und die Guete aus dem Quelle-DatenSatz ins Ziel.
	 *
	 * @param quelle
	 *            Quelle-Datebsatz
	 * @param ziel
	 *            Ziel-Datensatz
	 */
	public void copyAttribute(final Data quelle, final Data ziel) {

		ziel.getItem(AbstraktAggregator.STR_GUETE).getUnscaledValue(AbstraktAggregator.STR_INDEX).set(quelle
				.getItem(AbstraktAggregator.STR_GUETE).getUnscaledValue(AbstraktAggregator.STR_INDEX).longValue());
		ziel.getItem(AbstraktAggregator.STR_GUETE).getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).set(quelle
				.getItem(AbstraktAggregator.STR_GUETE).getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).longValue());
		ziel.getUnscaledValue(AbstraktAggregator.STR_WERT)
		.set(quelle.getUnscaledValue(AbstraktAggregator.STR_WERT).longValue());
	}

	protected LZZDaten getLzzDatenForSystemObject(final SystemObject object) {
		return mapSoLzzZumLZZDaten.get(object);
	}

	protected void resetLzzDaten() {
		mapSoLzzZumLZZDaten.clear();
	}

	protected LZZDaten addLzzDaten(final SystemObject object, final LZZDaten daten) {
		return mapSoLzzZumLZZDaten.put(object, daten);
	}

	protected DataDescription getDdAggregierteLzd() {
		return ddAggregierteLzd;
	}
}
