/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.dataIdentificationSettings.DataIdentification;

/**
 * Diese Klasse ist die Basis fuer die Klassen Axl Und Lve DeVerwaltung enthaelt
 * eine gemeinsame Schnittstelle fuer beiden Klassen und die gemeinsamen
 * Variablen.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public abstract class AbstraktDeVerwaltung implements SBArchivAnfrageBearbeiter {

	/**
	 * Verbindung zum Datenverteiler.
	 */
	private final ClientDavInterface dav;
	/**
	 * Die HauptKlasse des Moduls.
	 */
	private final AbfragePufferdaten verwaltungHaupt;
	/**
	 * Liste der bearbeitenden SteuerModule, enthaelt alle benoetigte informationen.
	 */
	private final Collection<Station> steuerModule;

	/**
	 * Standard Konstruktor.
	 *
	 * @param verwaltungHaupt
	 *            Die HauptKlasse des AbfrageModuls AbfragePufferdaten
	 * @param steuerModule
	 *            Menge der SteuerModule, die bearbeitet sollen
	 */
	public AbstraktDeVerwaltung(final AbfragePufferdaten verwaltungHaupt, final Collection<Station> steuerModule) {
		this.verwaltungHaupt = verwaltungHaupt;
		this.dav = verwaltungHaupt.getDatenVerteiler();
		this.steuerModule = steuerModule;
	}

	/**
	 * Erste Anmeldung auf Sendung und Empfang von Daten vom Datenverteiler.
	 */
	public abstract void ersteAnmeldung();

	/**
	 * Anmeldung auf Sendung und Empfang von Daten vom Datenverteiler, bei jeder
	 * Archivparemetrierungsaenderung.
	 */
	public abstract void anmeldung();

	/**
	 * Abmeldung der Sendung und Empfang von Daten vom Datenverteiler.
	 */
	public abstract void abmeldung();

	/**
	 * Ein Datensatz mit Zeitstempel (AXL oder LVE) gekommen ist.
	 *
	 * @param datei
	 *            Daten
	 */
	public abstract void aktualisiereDeZeitStemepel(ResultData datei);

	/**
	 * Betriebsparameter Datensatz gekommen ist.
	 *
	 * @param datei
	 *            Daten
	 */
	public abstract void setBetriebsParameter(ResultData datei);

	/**
	 * Generiert eine DatenAbfrage fuer die DEs der Station.
	 *
	 * @param st
	 *            Station
	 */
	public abstract void generiereAbfrage(Station st);

	/**
	 * Generiert eine DatenAbfrage.
	 *
	 * @param systemDe
	 *            Das Objekt, an dem die Abfrage geschick sein soll
	 * @param zeitStempel
	 *            Zeitstemepl von
	 * @param anzahl
	 *            Anzahl der erwarteten Datensaetzen
	 */
	public abstract void generiereAbfrage(SystemObject systemDe, long zeitStempel, long anzahl);

	/**
	 * Ein Datensatz mit neuen Einstellungen gekommen ist - Neue Parametrierung.
	 *
	 * @param dateiID
	 *            Datenbeschreibung
	 * @param neueDatei
	 *            Daten
	 */
	public abstract void bearbeiteNeueEinstellung(DataIdentification dateiID, Data neueDatei);

	/**
	 * Ein Datensatz mit veraenderten Einstellungen gekommen ist - Umparametrierung.
	 *
	 * @param dateiID
	 *            Datenbeschreibung
	 * @param alteDatei
	 *            Daten der alten Einstellungen
	 * @param neueDatei
	 *            Daten der neuen Einstellungen
	 */
	public abstract void bearbeiteVeraenderteEinstellung(DataIdentification dateiID, Data alteDatei, Data neueDatei);

	/**
	 * Ein Datensatz mit geloeschten Einstellungen gekommen ist - Nicht mehr
	 * parametriert.
	 *
	 * @param dateiID
	 *            Datenbeschreibung
	 * @param alteDatei
	 *            Daten der alten Einstellung
	 */
	public abstract void bearbeiteGeloeschteEinstellung(DataIdentification dateiID, Data alteDatei);

	/**
	 * Eine Abfrage auf das Archiv generiert son soll.
	 */
	public abstract void streamBasierteArchivAbfrage();

	/**
	 * Aktualsiert das Status der Station.
	 *
	 * @param st
	 *            Station
	 * @param s
	 *            Ausgabestring, Ereignis bei dem die Statusenderung geschehen ist
	 */
	public abstract void aktualisiereStatus(Station st, String s);

	/**
	 * Wird gerufen wenn man eine Negative Quittung erhalten hat.
	 *
	 * @param datei
	 *            Empfangene Datei
	 */
	public abstract void bearbeiteNegativeQuittung(ResultData datei);

	/**
	 * Am Ende jeder Abfrage wird das Aggregationsmodul getriggert.
	 *
	 * @param st
	 *            Station die getriggert hat
	 * @param timeout
	 *            true, wenn es nach einem Timeout geschehen ist
	 */
	public void abfrageEnde(final Station st, final boolean timeout) {
		verwaltungHaupt.aggregiere();
	}

	/**
	 * Wegen Testzwecken wird ueberschrieben.
	 *
	 * @param st
	 *            Station fuer die Anfrage gemacht wird
	 * @param anzahl
	 *            Anzahl der Abgefragten DS
	 * @param jetzt
	 *            Zeitstempel
	 * @param timeout
	 *            Timeout
	 */
	public void abfrageAusgabe(final Station st, final long anzahl, final long jetzt, final long timeout) {
		// Standardimplementierung tut nichts
	}

	/**
	 * Wegen Testzwecken wird ueberschrieben.
	 *
	 * @param st
	 *            Station fuer die Anfrage gemacht wird
	 * @param s
	 *            Beschreibeung des Ereignises
	 */
	public void statusAktualisierungAusgabe(final Station st, final String s) {
		// Standardimplementierung tut nichts
	}

	/**
	 * Liefert dem Hauptmodul.
	 *
	 * @return AbfragepufferDaten Klasse
	 */
	public AbfragePufferdaten getHauptModul() {
		return this.verwaltungHaupt;
	}

	/**
	 * Wegen Test Zwecken wird es in vererbten Klassen ueberdeckt.
	 *
	 * @return Zeit
	 */
	public long currentTimeMillis() {
		return System.currentTimeMillis();
	}

	/**
	 * Wegen Test Zwecken wird es in vererbten Klassen ueberdeckt.
	 *
	 * @param intervall
	 *            Reeler Timeout Intervall
	 * @return Virtueller Intervall
	 */
	public long convertTimeoutIntervall(final long intervall) {
		return intervall;
	}

	protected AbfragePufferdaten getVerwaltungHaupt() {
		return verwaltungHaupt;
	}

	protected Collection<Station> getSteuerModule() {
		return steuerModule;
	}

	protected ClientDavInterface getDav() {
		return dav;
	}
}
