/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.guete.GWert;
import de.bsvrz.dua.guete.GueteException;
import de.bsvrz.dua.guete.GueteVerfahren;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Rechnet durschschnittliche Geschwindigkeit aus.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class AggregatorGeschwindigkeit extends AbstraktAggregator {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * ATG String Konstante.
	 */
	private static final String ATG_LZD_GESCHWINDIGKEIT = "atg.langZeitDatenGeschwindigkeit";

	/**
	 * Die Geschwindigkeiten in Aggregationsdatensatz sind skaliert.
	 */
	private static final int SKALIERUNG_KOEFF = 10;

	/**
	 * Erwitert die Klasse LZZDaten um den Anzahl der Fahrzeuge und gewichtete
	 * Summe der Geschwindigkeiten.
	 */
	private class LZZGeschwDaten extends LZZDaten {
		/**
		 * Summe der Fahzeuganzahl.
		 */
		private final long[] summeAnzahl = new long[AbstraktAggregator.FZ_ATTRIB.length];
		/**
		 * Summer der gewichteten Geschwindigkeiten.
		 */
		private final long[] summeGeschw = new long[AbstraktAggregator.FZ_ATTRIB.length];

		/**
		 * Standardkonstruktor.
		 */
		LZZGeschwDaten() {
			resetDaten();
		}

		/**
		 * Initialisiert die Daten.
		 */
		@Override
		public void resetDaten() {
			super.resetDaten();
			for (int i = 0; i < summeAnzahl.length; i++) {
				summeAnzahl[i] = 0;
				summeGeschw[i] = 0;
			}
		}

		/**
		 * Ergibt die Summe des Anzahls der FZ.
		 *
		 * @return die Summenwerte
		 */
		public long[] getSummeAnzahl() {
			return summeAnzahl;
		}

		/**
		 * Ergibt die Summe der Geschwindigkeit der FZ.
		 *
		 * @return die Summenwerte
		 */
		public long[] getSummeGeschw() {
			return summeGeschw;
		}
	}

	@Override
	public DataDescription init(final Aspect asp, final ClientDavInterface dav,
			final Collection<SystemObject> objekteLZZ) {

		resetLzzDaten();

		for (final SystemObject lzz : objekteLZZ) {
			final LZZGeschwDaten lzzStruct = new LZZGeschwDaten();
			final Data data = dav.createData(
					dav.getDataModel().getAttributeGroup(AggregatorGeschwindigkeit.ATG_LZD_GESCHWINDIGKEIT));
			lzzStruct.setErgebnissDatenSatz(data);
			setLeer(data);
			addLzzDaten(lzz, lzzStruct);
		}

		return new DataDescription(
				dav.getDataModel().getAttributeGroup(AggregatorGeschwindigkeit.ATG_LZD_GESCHWINDIGKEIT), asp);
	}

	@Override
	public void bearbeiteDatenSatz(final SystemObject objekt, final Data datensatz, final long zeitStempel) {

		if (datensatz == null) {
			return;
		}
		final LZZGeschwDaten lzz = (LZZGeschwDaten) getLzzDatenForSystemObject(objekt);
		if (lzz == null) {
			return;
		}

		final Data ergebnissDatensatz = lzz.getErgebnissDatenSatz();
		final long[] summe = lzz.getSummeAnzahl();
		final long[] summeGeschw = lzz.getSummeGeschw();

		lzz.incrementAnzahlDatenSaetze();
		for (int i = 0; i < AbstraktAggregator.FZ_ATTRIB.length; i++) {
			final Data geschwData = datensatz.getItem("v" + AbstraktAggregator.FZ_ATTRIB[i]);
			final Data anzahlData = datensatz.getItem("q" + AbstraktAggregator.FZ_ATTRIB[i]);
			final Data ergebnissData = ergebnissDatensatz.getItem(AbstraktAggregator.FZ_ATTRIB[i]);
			long anzahl = anzahlData.getUnscaledValue(AbstraktAggregator.STR_WERT).longValue();
			long wert = geschwData.getUnscaledValue(AbstraktAggregator.STR_WERT).longValue();
			final boolean[] ermittelbarWertEingekommen = lzz.getErmittelbarWert();

			if ((wert > 0) && (anzahl > 0)) {
				ermittelbarWertEingekommen[i] = true;
			}

			if (wert < 0) {
				wert = 0;
			}
			if (anzahl < 0) {
				anzahl = 0;
			}

			// summeGeschw[i] += anzahl * wert;
			summeGeschw[i] += wert;

			if (lzz.getAnzahlDatenSaetze() == 1) {
				copyAttribute(geschwData, ergebnissData);
			} else {
				try {
					final GWert summeGuete = GueteVerfahren.summe(
							new GWert(datensatz, "v" + AbstraktAggregator.FZ_ATTRIB[i]),
							new GWert(ergebnissDatensatz, AbstraktAggregator.FZ_ATTRIB[i]));
					summeGuete.exportiere(ergebnissData.getItem(AbstraktAggregator.STR_GUETE));
				} catch (final GueteException e) {
					AggregatorGeschwindigkeit.LOGGER
					.error("Fehler bei Ausrechnen der Guete fuer aggregierte Daten " + e.getMessage());
					e.printStackTrace();
				}
			}
			summe[i] += 1;

		}
	}

	@Override
	public void erzeugeErgebniss(final SystemObject objekt, final long zeitStempelVon, final long zeitStempelBis,
			final boolean ermittelbar) {

		final LZZGeschwDaten lzz = (LZZGeschwDaten) getLzzDatenForSystemObject(objekt);
		if (lzz == null) {
			return;
		}
		final Data ergebnissDatenSatz = lzz.getErgebnissDatenSatz();

		final long[] summe = lzz.getSummeAnzahl();
		final long[] summeGeschw = lzz.getSummeGeschw();
		final boolean[] ermittelbarWertEingekommen = lzz.getErmittelbarWert();

		if (lzz.getZeitStempelLetztesDS() >= zeitStempelVon) {
			setLeer(ergebnissDatenSatz);
			lzz.setAnzahlDatenSaetze(0);
			lzz.resetDaten();
			return;
		}

		for (int i = 0; i < AbstraktAggregator.FZ_ATTRIB.length; i++) {
			final Data subErgebnissData = ergebnissDatenSatz.getItem(AbstraktAggregator.FZ_ATTRIB[i]);
			if (ermittelbarWertEingekommen[i]) {
				summeGeschw[i] = (summeGeschw[i] * AggregatorGeschwindigkeit.SKALIERUNG_KOEFF) / summe[i];
				subErgebnissData.getUnscaledValue(AbstraktAggregator.STR_WERT).set(summeGeschw[i]);
			} else {
				setAttributNichtErmittelbar(subErgebnissData);
			}
		}

		if (!ermittelbar) {
			setNichtErmittelbar(ergebnissDatenSatz);
		}

		final ResultData result = new ResultData(objekt, getDdAggregierteLzd(), zeitStempelVon, ergebnissDatenSatz,
				true);

		sendeDatenSatz(objekt, result);
		setLeer(ergebnissDatenSatz);
		lzz.setAnzahlDatenSaetze(0);
		lzz.resetDaten();
		lzz.setZeitStempelLetztesDS(zeitStempelVon);
	}
}
