/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.guete.GWert;
import de.bsvrz.dua.guete.GueteException;
import de.bsvrz.dua.guete.GueteVerfahren;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Ausrechnet Die Vxx Geschwindigkeit ( die Geschwindigkeit, die von xx
 * Perzentill der Fahrzeugen unterschritten wurde ).
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class AggregatorVxx extends AggregatorMitHistorie {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * ATG String Konstante.
	 */
	private static final String ATG_LZD_V85 = "atg.langZeitDatenV85";
	private static final String ATG_LZD_V50 = "atg.langZeitDatenV50";
	private static final String ATG_LZD_V15 = "atg.langZeitDatenV15";

	/**
	 * Die Geschwindigkeit in dem Vxx Datensatz sit skaliert mit Koeffizient 10.
	 */
	private static final int SKALIERUNG_KOEFF = 10;

	/**
	 * Diese statische variable ermoeglicht das Benehmen des Moduls zu aendern.
	 * Die V85 geschwindigkeit ist in LZD enthalten, so wir koennen es aus
	 * dieser Information nur mitteln, sonst koennen wir es mit dem selben
	 * Algorithmus ausrechnen, wie bei anderen Perzentilen.
	 */
	private static final boolean PERZENTIL85_AUS_LZD = false;

	/**
	 * Der geawehlte Perzentil, der im Konstruktor gegeben wurde.
	 */
	private String atgLzdVxx;

	/**
	 * Nummerische Wert des Perzentils.
	 */
	private final long perzentill;

	/**
	 * Erweiterung der LZZDaten Klasse enthaelt den aufsumierten Anzahl der
	 * Fahrzeuge und aud die gewichtete Summe der V85 Geschwindigkeiten.
	 */
	private class LZZVxxDaten extends LZZHistDaten {
		/**
		 * Summe des Anzahls der Fahrzeuge.
		 */
		private final long[] summeAnzahl = new long[AbstraktAggregator.FZ_ATTRIB.length];
		/**
		 * Gewichtete Summe der V85 geschwindigkeit.
		 */
		private final long[] summeV85 = new long[AbstraktAggregator.FZ_ATTRIB.length];

		/**
		 * Standardkonstruktor.
		 */
		LZZVxxDaten() {
			resetDaten();
		}

		/**
		 * Ergibt die v85 Summe.
		 *
		 * @return Summe der V85 Geschwindigkeiten
		 */
		public long[] getSummeV85() {
			return summeV85;
		}

		/**
		 * Initialisiert die Daten.
		 */
		@Override
		public void resetDaten() {
			for (int i = 0; i < summeAnzahl.length; i++) {
				summeAnzahl[i] = 0;
				summeV85[i] = 0;
			}
			super.setAnzahlDatenSaetze(0);
			super.clearHistorie();
		}

		/**
		 * Ergibt die v85 Summe.
		 *
		 * @return Summe der V85 Geschwindigkeiten
		 */
		public long[] getSummeAnzahl() {
			return summeAnzahl;
		}
	}

	/**
	 * Standardkonsturktor.
	 *
	 * @param xx
	 *            Der Perzentill der Fahrzeuge
	 */
	public AggregatorVxx(final int xx) {
		switch (xx) {
		case 85:
			atgLzdVxx = AggregatorVxx.ATG_LZD_V85;
			break;
		case 50:
			atgLzdVxx = AggregatorVxx.ATG_LZD_V50;
			break;
		case 15:
			atgLzdVxx = AggregatorVxx.ATG_LZD_V15;
			break;
		default:
			AggregatorVxx.LOGGER.error("Unbekante Langzeitdatenart Gschwindigkeit-V" + xx);
		}
		perzentill = xx;
	}

	@Override
	public void bearbeiteDatenSatz(final SystemObject objekt, final Data datensatz, final long zeitStempel) {

		if (datensatz == null) {
			return;
		}
		final LZZVxxDaten lzz = (LZZVxxDaten) getLzzDatenForSystemObject(objekt);
		if (lzz == null) {
			return;
		}
		long anzahl;
		final long[] summe = lzz.getSummeAnzahl();
		final long[] summeV85 = lzz.getSummeV85();
		final Data ergebnissData = lzz.getErgebnissDatenSatz();

		lzz.incrementAnzahlDatenSaetze();
		lzz.addDatenSatzZurHistorie(datensatz);

		if ((perzentill == 85) && !AggregatorVxx.PERZENTIL85_AUS_LZD) {
			for (int i = 0; i < AbstraktAggregator.FZ_ATTRIB.length; i++) {
				final Data subV85Data = datensatz.getItem("v85" + AbstraktAggregator.FZ_ATTRIB[i]);
				final Data subAnzahlData = datensatz.getItem("q" + AbstraktAggregator.FZ_ATTRIB[i]);
				final Data ergebnissSubData = ergebnissData.getItem(AbstraktAggregator.FZ_ATTRIB[i]);

				anzahl = subAnzahlData.getUnscaledValue(AbstraktAggregator.STR_WERT).longValue();
				long wert = subV85Data.getUnscaledValue(AbstraktAggregator.STR_WERT).longValue();

				if (wert < 0) {
					wert = 0;
				}
				if (anzahl < 0) {
					anzahl = 0;
				}

				summeV85[i] += anzahl * wert;

				if (lzz.getAnzahlDatenSaetze() == 1) {
					copyAttribute(subV85Data, ergebnissSubData);
				} else {
					try {
						final GWert summeGuete = GueteVerfahren.summe(
								GueteVerfahren.gewichte(new GWert(datensatz, "v85" + AbstraktAggregator.FZ_ATTRIB[i]),
										summe[i]),
								GueteVerfahren.gewichte(new GWert(ergebnissData, AbstraktAggregator.FZ_ATTRIB[i]),
										anzahl));
						summeGuete.exportiere(ergebnissSubData.getItem(AbstraktAggregator.STR_GUETE));
					} catch (final GueteException e) {
						AggregatorVxx.LOGGER
						.error("Fehler bei Ausrechnen der Guete fuer aggregierte Daten " + e.getMessage());
						e.printStackTrace();
					}
				}
				summe[i] += anzahl;

			}
		} else {
			for (int i = 0; i < AbstraktAggregator.FZ_ATTRIB.length; i++) {
				final Data subData = datensatz.getItem("q" + AbstraktAggregator.FZ_ATTRIB[i]);
				summe[i] += subData.getUnscaledValue(AbstraktAggregator.STR_WERT).longValue();
			}
		}
	}

	@Override
	public void erzeugeErgebniss(final SystemObject objekt, final long zeitStempelVon, final long zeitStempelBis,
			final boolean ermittelbar) {

		final LZZVxxDaten lzz = (LZZVxxDaten) getLzzDatenForSystemObject(objekt);
		final Data ergebnissDatenSatz = lzz.getErgebnissDatenSatz();

		if (lzz.getZeitStempelLetztesDS() >= zeitStempelVon) {
			setLeer(ergebnissDatenSatz);
			lzz.resetDaten();
			return;
		}
		final List<Data> alleDs = lzz.getHistorie();

		final long[] summeTotal = lzz.getSummeAnzahl();
		final long[] summeV85 = lzz.getSummeV85();

		// V85 Daten Haben wir schon im LZD, es ist genuegend nur dem mittelwert
		// zu bestimmen
		// andere muessen wir ausrechnen
		if ((perzentill == 85) && !AggregatorVxx.PERZENTIL85_AUS_LZD) {

			for (int i = 0; i < AbstraktAggregator.FZ_ATTRIB.length; i++) {
				final Data subErgebnissData = ergebnissDatenSatz.getItem(AbstraktAggregator.FZ_ATTRIB[i]);
				if (summeTotal[i] != 0) {
					summeV85[i] = (summeV85[i] * AggregatorVxx.SKALIERUNG_KOEFF) / summeTotal[i];
					subErgebnissData.getUnscaledValue(AbstraktAggregator.STR_WERT).set(summeV85[i]);
				} else {
					setAttributNichtErmittelbar(subErgebnissData);
				}
			}
		} else {
			for (int i = 0; i < AbstraktAggregator.FZ_ATTRIB.length; i++) {
				Collections.sort(alleDs, new DatenSatzVergleicher("v" + AbstraktAggregator.FZ_ATTRIB[i]));

				long summeTeilweise = 0;

				for (final Data einDs : alleDs) {
					final long wert = einDs.getItem("q" + AbstraktAggregator.FZ_ATTRIB[i])
							.getUnscaledValue(AbstraktAggregator.STR_WERT).longValue();
					if (wert >= 0) {
						summeTeilweise += wert;
						if ((summeTeilweise * 100) >= (summeTotal[i] * perzentill)) {
							copyAttribute(einDs.getItem("v" + AbstraktAggregator.FZ_ATTRIB[i]),
									ergebnissDatenSatz.getItem(AbstraktAggregator.FZ_ATTRIB[i]));
							break;
						}
					}
				}
			}
		}

		if (!ermittelbar) {
			setNichtErmittelbar(ergebnissDatenSatz);
		}

		final ResultData result = new ResultData(objekt, getDdAggregierteLzd(), zeitStempelVon, ergebnissDatenSatz,
				true);
		sendeDatenSatz(objekt, result);
		setLeer(ergebnissDatenSatz);

		lzz.resetDaten();
		lzz.setZeitStempelLetztesDS(zeitStempelVon);
	}

	@Override
	public DataDescription init(final Aspect asp, final ClientDavInterface dav,
			final Collection<SystemObject> objekteLZZ) {

		resetLzzDaten();

		final DataDescription dd = new DataDescription(dav.getDataModel().getAttributeGroup(atgLzdVxx), asp);

		for (final SystemObject lzz : objekteLZZ) {
			final LZZDaten lzzStruct = new LZZVxxDaten();
			final Data data = dav.createData(dav.getDataModel().getAttributeGroup(atgLzdVxx));
			lzzStruct.setErgebnissDatenSatz(data);
			setLeer(data);
			addLzzDaten(lzz, lzzStruct);
		}
		return dd;
	}

}
